@@ -43,16 +43,19 @@
 #include <packagekit-glib2/pk-version.h>
 #include <polkit/polkit.h>
 
-#include "pk-cleanup.h"
 #include "pk-backend.h"
 #include "pk-dbus.h"
 #include "pk-engine.h"
-#include "pk-network.h"
 #include "pk-shared.h"
 #include "pk-transaction-db.h"
 #include "pk-transaction.h"
 #include "pk-scheduler.h"
 
+#ifndef glib_autoptr_cleanup_PolkitAuthorizationResult
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(PolkitAuthorizationResult, g_object_unref)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(PolkitSubject, g_object_unref)
+#endif
+
 static void     pk_engine_finalize	(GObject       *object);
 static void	pk_engine_set_locked (PkEngine *engine, gboolean is_locked);
 
@@ -72,12 +75,13 @@ struct PkEnginePrivate
 	PkScheduler		*scheduler;
 	PkTransactionDb		*transaction_db;
 	PkBackend		*backend;
-	PkNetwork		*network;
+	GNetworkMonitor		*network_monitor;
 	GKeyFile		*conf;
 	PkDbus			*dbus;
 	GFileMonitor		*monitor_conf;
 	GFileMonitor		*monitor_binary;
 	GFileMonitor		*monitor_offline;
+	GFileMonitor		*monitor_offline_upgrade;
 	PkBitfield		 roles;
 	PkBitfield		 groups;
 	PkBitfield		 filters;
@@ -116,14 +120,7 @@ gboolean pk_engine_filter_check (const gchar *filter, GError **error);
  * pk_engine_error_quark:
  * Return value: Our personal error quark.
  **/
-GQuark
-pk_engine_error_quark (void)
-{
-	static GQuark quark = 0;
-	if (!quark)
-		quark = g_quark_from_static_string ("pk_engine_error");
-	return quark;
-}
+G_DEFINE_QUARK (pk-engine-error-quark, pk_engine_error)
 
 /**
  * pk_engine_error_get_type:
@@ -183,7 +180,7 @@ pk_engine_set_inhibited (PkEngine *engine, gboolean inhibited)
 static void
 pk_engine_scheduler_changed_cb (PkScheduler *scheduler, PkEngine *engine)
 {
-	_cleanup_strv_free_ gchar **transaction_list = NULL;
+	g_auto(GStrv) transaction_list = NULL;
 
 	g_return_if_fail (PK_IS_ENGINE (engine));
 
@@ -278,11 +275,9 @@ static void
 pk_engine_inhibit (PkEngine *engine)
 {
 #ifdef HAVE_SYSTEMD
-	const gint *fd_list;
-	gint fd_list_len = 0;
-	_cleanup_error_free_ GError *error = NULL;
-	_cleanup_object_unref_ GUnixFDList *out_fd_list = NULL;
-	_cleanup_variant_unref_ GVariant *res = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GUnixFDList) out_fd_list = NULL;
+	g_autoptr(GVariant) res = NULL;
 
 	/* already inhibited */
 	if (engine->priv->logind_fd != 0)
@@ -314,12 +309,11 @@ pk_engine_inhibit (PkEngine *engine)
 	}
 
 	/* keep fd as cookie */
-	fd_list = g_unix_fd_list_peek_fds (out_fd_list, &fd_list_len);
-	if (fd_list_len != 1) {
+	if (g_unix_fd_list_get_length (out_fd_list) != 1) {
 		g_warning ("invalid response from logind");
 		return;
 	}
-	engine->priv->logind_fd = fd_list[0];
+	engine->priv->logind_fd = g_unix_fd_list_get (out_fd_list, 0, NULL);
 	g_debug ("opened logind fd %i", engine->priv->logind_fd);
 #endif
 }
@@ -402,9 +396,8 @@ pk_engine_backend_updates_changed_cb (PkBackend *backend, PkEngine *engine)
 static gboolean
 pk_engine_state_changed_cb (gpointer data)
 {
-	PkNetworkEnum state;
 	PkEngine *engine = PK_ENGINE (data);
-	_cleanup_error_free_ GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 
 	g_return_val_if_fail (PK_IS_ENGINE (engine), FALSE);
 
@@ -413,8 +406,7 @@ pk_engine_state_changed_cb (gpointer data)
 		g_warning ("failed to invalidate: %s", error->message);
 
 	/* if network is not up, then just reschedule */
-	state = pk_network_get_network_state (engine->priv->network);
-	if (state == PK_NETWORK_ENUM_OFFLINE) {
+	if (!g_network_monitor_get_network_available (engine->priv->network_monitor)) {
 		/* wait another timeout of PK_ENGINE_STATE_CHANGED_x_TIMEOUT */
 		return TRUE;
 	}
@@ -499,7 +491,7 @@ pk_engine_set_proxy_internal (PkEngine *engine, const gchar *sender,
 {
 	gboolean ret = FALSE;
 	guint uid;
-	_cleanup_free_ gchar *session = NULL;
+	g_autofree gchar *session = NULL;
 
 	/* get uid */
 	uid = pk_dbus_get_uid (engine->priv->dbus, sender);
@@ -555,8 +547,8 @@ pk_engine_action_obtain_proxy_authorization_finished_cb (PolkitAuthority *author
 	GError *error;
 	gboolean ret;
 	PkEnginePrivate *priv = state->engine->priv;
-	_cleanup_error_free_ GError *error_local = NULL;
-	_cleanup_object_unref_ PolkitAuthorizationResult *result = NULL;
+	g_autoptr(GError) error_local = NULL;
+	g_autoptr(PolkitAuthorizationResult) result = NULL;
 
 	/* finish the call */
 	result = polkit_authority_check_authorization_finish (priv->authority, res, &error_local);
@@ -627,13 +619,13 @@ pk_engine_is_proxy_unchanged (PkEngine *engine, const gchar *sender,
 {
 	guint uid;
 	gboolean ret = FALSE;
-	_cleanup_free_ gchar *session = NULL;
-	_cleanup_free_ gchar *proxy_http_tmp = NULL;
-	_cleanup_free_ gchar *proxy_https_tmp = NULL;
-	_cleanup_free_ gchar *proxy_ftp_tmp = NULL;
-	_cleanup_free_ gchar *proxy_socks_tmp = NULL;
-	_cleanup_free_ gchar *no_proxy_tmp = NULL;
-	_cleanup_free_ gchar *pac_tmp = NULL;
+	g_autofree gchar *session = NULL;
+	g_autofree gchar *proxy_http_tmp = NULL;
+	g_autofree gchar *proxy_https_tmp = NULL;
+	g_autofree gchar *proxy_ftp_tmp = NULL;
+	g_autofree gchar *proxy_socks_tmp = NULL;
+	g_autofree gchar *no_proxy_tmp = NULL;
+	g_autofree gchar *pac_tmp = NULL;
 
 	/* get uid */
 	uid = pk_dbus_get_uid (engine->priv->dbus, sender);
@@ -691,7 +683,7 @@ pk_engine_set_proxy (PkEngine *engine,
 	gboolean ret;
 	const gchar *sender;
 	PkEngineDbusState *state;
-	_cleanup_object_unref_ PolkitSubject *subject = NULL;
+	g_autoptr(PolkitSubject) subject = NULL;
 
 	g_return_if_fail (PK_IS_ENGINE (engine));
 
@@ -779,8 +771,8 @@ pk_engine_can_authorize_action_id (PkEngine *engine,
 				   const gchar *sender,
 				   GError **error)
 {
-	_cleanup_object_unref_ PolkitAuthorizationResult *res = NULL;
-	_cleanup_object_unref_ PolkitSubject *subject = NULL;
+	g_autoptr(PolkitAuthorizationResult) res = NULL;
+	g_autoptr(PolkitSubject) subject = NULL;
 
 	/* check subject */
 	subject = polkit_system_bus_name_new (sender);
@@ -879,18 +871,52 @@ pk_engine_offline_file_changed_cb (GFileMonitor *file_monitor,
 						 g_variant_new_boolean (ret));
 }
 
+/**
+ * pk_engine_offline_upgrade_file_changed_cb:
+ **/
+static void
+pk_engine_offline_upgrade_file_changed_cb (GFileMonitor *file_monitor,
+                                           GFile *file, GFile *other_file,
+                                           GFileMonitorEvent event_type,
+                                           PkEngine *engine)
+{
+	gboolean ret;
+	g_return_if_fail (PK_IS_ENGINE (engine));
+
+	ret = g_file_test (PK_OFFLINE_PREPARED_UPGRADE_FILENAME, G_FILE_TEST_EXISTS);
+	pk_engine_emit_offline_property_changed (engine,
+						 "UpgradePrepared",
+						 g_variant_new_boolean (ret));
+}
+
+/**
+ * pk_engine_get_network_state:
+ **/
+static PkNetworkEnum
+pk_engine_get_network_state (GNetworkMonitor *network_monitor)
+{
+	if (!g_network_monitor_get_network_available (network_monitor))
+		return PK_NETWORK_ENUM_OFFLINE;
+	/* this isn't exactly true, but it's what the UI expects */
+	if (g_network_monitor_get_network_metered (network_monitor))
+		return PK_NETWORK_ENUM_MOBILE;
+	return PK_NETWORK_ENUM_ONLINE;
+}
+
 /**
  * pk_engine_network_state_changed_cb:
  **/
 static void
-pk_engine_network_state_changed_cb (PkNetwork *network, PkNetworkEnum network_state, PkEngine *engine)
+pk_engine_network_state_changed_cb (GNetworkMonitor *network_monitor,
+				    gboolean available,
+				    PkEngine *engine)
 {
+	PkNetworkEnum network_state;
 	g_return_if_fail (PK_IS_ENGINE (engine));
 
-	/* already set */
-	if (engine->priv->network_state == network_state)
+	network_state = pk_engine_get_network_state (network_monitor);
+	if (network_state == engine->priv->network_state)
 		return;
-
 	engine->priv->network_state = network_state;
 
 	/* emit */
@@ -906,9 +932,9 @@ static void
 pk_engine_setup_file_monitors (PkEngine *engine)
 {
 	const gchar *filename = "/etc/PackageKit/PackageKit.conf";
-	_cleanup_error_free_ GError *error = NULL;
-	_cleanup_object_unref_ GFile *file_binary = NULL;
-	_cleanup_object_unref_ GFile *file_conf = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GFile) file_binary = NULL;
+	g_autoptr(GFile) file_conf = NULL;
 
 	/* monitor the binary file for changes */
 	file_binary = g_file_new_for_path (LIBEXECDIR "/packagekitd");
@@ -940,13 +966,23 @@ pk_engine_setup_file_monitors (PkEngine *engine)
 
 	/* set up the prepared update monitor */
 	engine->priv->monitor_offline = pk_offline_get_prepared_monitor (NULL, &error);
-	if (engine->priv->introspection == NULL) {
+	if (engine->priv->monitor_offline == NULL) {
 		g_warning ("Failed to set watch on %s: %s",
 			   PK_OFFLINE_PREPARED_FILENAME, error->message);
 		return;
 	}
 	g_signal_connect (engine->priv->monitor_offline, "changed",
 			  G_CALLBACK (pk_engine_offline_file_changed_cb), engine);
+
+	/* set up the prepared system upgrade monitor */
+	engine->priv->monitor_offline_upgrade = pk_offline_get_prepared_upgrade_monitor (NULL, &error);
+	if (engine->priv->monitor_offline_upgrade == NULL) {
+		g_warning ("Failed to set watch on %s: %s",
+			   PK_OFFLINE_PREPARED_UPGRADE_FILENAME, error->message);
+		return;
+	}
+	g_signal_connect (engine->priv->monitor_offline_upgrade, "changed",
+			  G_CALLBACK (pk_engine_offline_upgrade_file_changed_cb), engine);
 }
 
 /**
@@ -1014,6 +1050,27 @@ pk_engine_offline_get_property (GDBusConnection *connection_, const gchar *sende
 		return g_variant_new_boolean (ret);
 	}
 
+	/* stat the file */
+	if (g_strcmp0 (property_name, "UpgradePrepared") == 0) {
+		gboolean ret;
+		ret = g_file_test (PK_OFFLINE_PREPARED_UPGRADE_FILENAME, G_FILE_TEST_EXISTS);
+		return g_variant_new_boolean (ret);
+	}
+
+	/* look at the symlink target */
+	if (g_strcmp0 (property_name, "UpdateTriggered") == 0) {
+		g_autofree gchar *link = NULL;
+		link = g_file_read_link (PK_OFFLINE_TRIGGER_FILENAME, NULL);
+		return g_variant_new_boolean (g_strcmp0 (link, PK_OFFLINE_PREPARED_FILENAME) == 0);
+	}
+
+	/* look at the symlink target */
+	if (g_strcmp0 (property_name, "UpgradeTriggered") == 0) {
+		g_autofree gchar *link = NULL;
+		link = g_file_read_link (PK_OFFLINE_TRIGGER_FILENAME, NULL);
+		return g_variant_new_boolean (g_strcmp0 (link, PK_OFFLINE_PREPARED_UPGRADE_FILENAME) == 0);
+	}
+
 	/* return an error */
 	g_set_error (error,
 		     PK_ENGINE_ERROR,
@@ -1154,10 +1211,10 @@ pk_engine_get_package_history (PkEngine *engine,
 	GVariantBuilder builder;
 	GVariant *value = NULL;
 	PkTransactionPast *item;
-	_cleanup_hashtable_unref_ GHashTable *deduplicate_hash = NULL;
-	_cleanup_hashtable_unref_ GHashTable *pkgname_hash = NULL;
-	_cleanup_list_free_ GList *keys = NULL;
-	_cleanup_object_unref_ PkPackage *package_tmp = NULL;
+	g_autoptr(GHashTable) deduplicate_hash = NULL;
+	g_autoptr(GHashTable) pkgname_hash = NULL;
+	g_autoptr(GList) keys = NULL;
+	g_autoptr(PkPackage) package_tmp = NULL;
 
 	list = pk_transaction_db_get_list (engine->priv->transaction_db, max_size);
 
@@ -1169,7 +1226,7 @@ pk_engine_get_package_history (PkEngine *engine,
 	deduplicate_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	package_tmp = pk_package_new ();
 	for (l = list; l != NULL; l = l->next) {
-		_cleanup_strv_free_ gchar **package_lines = NULL;
+		g_auto(GStrv) package_lines = NULL;
 		item = PK_TRANSACTION_PAST (l->data);
 
 		/* ignore anything that failed */
@@ -1182,8 +1239,15 @@ pk_engine_get_package_history (PkEngine *engine,
 			continue;
 		package_lines = g_strsplit (data, "\n", -1);
 		for (i = 0; package_lines[i] != NULL; i++) {
-			ret = pk_package_parse (package_tmp, package_lines[i], error);
-			g_assert (ret);
+			g_autoptr(GError) error_local = NULL;
+			ret = pk_package_parse (package_tmp,
+						package_lines[i],
+						&error_local);
+			if (!ret) {
+				g_warning ("Failed to parse package: '%s': %s",
+					   package_lines[i], error_local->message);
+				continue;
+			}
 
 			/* not the package we care about */
 			if (!pk_engine_package_name_in_strv (package_names, package_tmp))
@@ -1272,9 +1336,9 @@ pk_engine_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 	gchar **package_names;
 	guint size;
 	gboolean is_priority = TRUE;
-	_cleanup_error_free_ GError *error = NULL;
-	_cleanup_free_ gchar *data = NULL;
-	_cleanup_strv_free_ gchar **array = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autofree gchar *data = NULL;
+	g_auto(GStrv) array = NULL;
 
 	g_return_if_fail (PK_IS_ENGINE (engine));
 	g_return_if_fail (pk_is_thread_default ());
@@ -1348,6 +1412,7 @@ pk_engine_daemon_method_call (GDBusConnection *connection_, const gchar *sender,
 	if (g_strcmp0 (method_name, "GetTransactionList") == 0) {
 		transaction_list = pk_scheduler_get_array (engine->priv->scheduler);
 		value = g_variant_new ("(^a&o)", transaction_list);
+		g_free (transaction_list);
 		g_dbus_method_invocation_return_value (invocation, value);
 		return;
 	}
@@ -1471,6 +1536,7 @@ typedef enum {
 	PK_ENGINE_OFFLINE_ROLE_CANCEL,
 	PK_ENGINE_OFFLINE_ROLE_CLEAR_RESULTS,
 	PK_ENGINE_OFFLINE_ROLE_TRIGGER,
+	PK_ENGINE_OFFLINE_ROLE_TRIGGER_UPGRADE,
 	PK_ENGINE_OFFLINE_ROLE_LAST
 } PkEngineOfflineRole;
 
@@ -1501,9 +1567,10 @@ pk_engine_offline_helper_cb (GObject *source, GAsyncResult *res, gpointer user_d
 	PkEngineOfflineAsyncHelper *helper = (PkEngineOfflineAsyncHelper *) user_data;
 	PkOfflineAction action;
 	gboolean ret;
-	_cleanup_error_free_ GError *error = NULL;
-	_cleanup_object_unref_ GFile *file = NULL;
-	_cleanup_object_unref_ PolkitAuthorizationResult *result = NULL;
+	g_autofree gchar *link = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GFile) file = NULL;
+	g_autoptr(PolkitAuthorizationResult) result = NULL;
 
 	/* finish the call */
 	result = polkit_authority_check_authorization_finish (POLKIT_AUTHORITY (source), res, &error);
@@ -1537,6 +1604,9 @@ pk_engine_offline_helper_cb (GObject *source, GAsyncResult *res, gpointer user_d
 	case PK_ENGINE_OFFLINE_ROLE_TRIGGER:
 		ret = pk_offline_auth_trigger (helper->action, &error);
 		break;
+	case PK_ENGINE_OFFLINE_ROLE_TRIGGER_UPGRADE:
+		ret = pk_offline_auth_trigger_upgrade (helper->action, &error);
+		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -1555,6 +1625,14 @@ pk_engine_offline_helper_cb (GObject *source, GAsyncResult *res, gpointer user_d
 						 "TriggerAction",
 						 g_variant_new_string (pk_offline_action_to_string (action)));
 
+	link = g_file_read_link (PK_OFFLINE_TRIGGER_FILENAME, NULL);
+	pk_engine_emit_offline_property_changed (helper->engine,
+						 "UpdateTriggered",
+						 g_variant_new_boolean (g_strcmp0 (link, PK_OFFLINE_PREPARED_FILENAME) == 0));
+	pk_engine_emit_offline_property_changed (helper->engine,
+						 "UpgradeTriggered",
+						 g_variant_new_boolean (g_strcmp0 (link, PK_OFFLINE_PREPARED_UPGRADE_FILENAME) == 0));
+
 	g_dbus_method_invocation_return_value (helper->invocation, NULL);
 	pk_engine_offline_helper_free (helper);
 }
@@ -1570,8 +1648,8 @@ pk_engine_offline_method_call (GDBusConnection *connection_, const gchar *sender
 {
 	PkEngine *engine = PK_ENGINE (user_data);
 	PkEngineOfflineAsyncHelper *helper;
-	_cleanup_error_free_ GError *error = NULL;
-	_cleanup_object_unref_ PolkitSubject *subject = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(PolkitSubject) subject = NULL;
 
 	g_return_if_fail (PK_IS_ENGINE (engine));
 
@@ -1636,8 +1714,35 @@ pk_engine_offline_method_call (GDBusConnection *connection_, const gchar *sender
 						      helper);
 		return;
 	}
+	if (g_strcmp0 (method_name, "TriggerUpgrade") == 0) {
+		const gchar *tmp;
+		PkOfflineAction action;
+		g_variant_get (parameters, "(&s)", &tmp);
+		action = pk_offline_action_from_string (tmp);
+		if (action == PK_OFFLINE_ACTION_UNKNOWN) {
+			g_dbus_method_invocation_return_error (invocation,
+							       PK_ENGINE_ERROR,
+							       PK_ENGINE_ERROR_NOT_SUPPORTED,
+							       "action %s unsupported",
+							       tmp);
+			return;
+		}
+		helper = g_new0 (PkEngineOfflineAsyncHelper, 1);
+		helper->engine = g_object_ref (engine);
+		helper->role = PK_ENGINE_OFFLINE_ROLE_TRIGGER_UPGRADE;
+		helper->invocation = g_object_ref (invocation);
+		helper->action = action;
+		polkit_authority_check_authorization (engine->priv->authority, subject,
+						      "org.freedesktop.packagekit.trigger-offline-update",
+						      NULL,
+						      POLKIT_CHECK_AUTHORIZATION_FLAGS_ALLOW_USER_INTERACTION,
+						      NULL,
+						      pk_engine_offline_helper_cb,
+						      helper);
+		return;
+	}
 	if (g_strcmp0 (method_name, "GetPrepared") == 0) {
-		_cleanup_strv_free_ gchar **package_ids = NULL;
+		g_auto(GStrv) package_ids = NULL;
 		GVariant *value = NULL;
 
 		package_ids = pk_offline_get_prepared_ids (&error);
@@ -1652,7 +1757,7 @@ pk_engine_offline_method_call (GDBusConnection *connection_, const gchar *sender
 		if (package_ids != NULL) {
 			value = g_variant_new ("(^as)", package_ids);
 		} else {
-			value = g_variant_new ("(as)");
+			value = g_variant_new ("(as)", NULL);
 		}
 		g_dbus_method_invocation_return_value (invocation, value);
 		return;
@@ -1668,7 +1773,7 @@ pk_engine_proxy_logind_cb (GObject *source_object,
 			   GAsyncResult *res,
 			   gpointer user_data)
 {
-	_cleanup_error_free_ GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	PkEngine *engine = PK_ENGINE (user_data);
 
 	engine->priv->logind_proxy = g_dbus_proxy_new_finish (res, &error);
@@ -1765,8 +1870,8 @@ pk_engine_on_name_lost_cb (GDBusConnection *connection_,
 static void
 pk_engine_init (PkEngine *engine)
 {
-	_cleanup_error_free_ GError *error = NULL;
-	_cleanup_free_ gchar *filename = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autofree gchar *filename = NULL;
 
 	engine->priv = PK_ENGINE_GET_PRIVATE (engine);
 
@@ -1784,10 +1889,10 @@ pk_engine_init (PkEngine *engine)
 	pk_directory_remove_contents (filename);
 
 	/* proxy the network state */
-	engine->priv->network = pk_network_new ();
-	g_signal_connect (engine->priv->network, "state-changed",
+	engine->priv->network_monitor = g_network_monitor_get_default ();
+	g_signal_connect (engine->priv->network_monitor, "network-changed",
 			  G_CALLBACK (pk_engine_network_state_changed_cb), engine);
-	engine->priv->network_state = pk_network_get_network_state (engine->priv->network);
+	engine->priv->network_state = pk_engine_get_network_state (engine->priv->network_monitor);
 
 	/* try to get the distro id */
 	engine->priv->distro_id = pk_get_distro_id ();
@@ -1871,9 +1976,9 @@ pk_engine_finalize (GObject *object)
 	g_object_unref (engine->priv->monitor_conf);
 	g_object_unref (engine->priv->monitor_binary);
 	g_object_unref (engine->priv->monitor_offline);
+	g_object_unref (engine->priv->monitor_offline_upgrade);
 	g_object_unref (engine->priv->scheduler);
 	g_object_unref (engine->priv->transaction_db);
-	g_object_unref (engine->priv->network);
 	if (engine->priv->authority != NULL)
 		g_object_unref (engine->priv->authority);
 	g_object_unref (engine->priv->backend);
