@@ -42,7 +42,6 @@
 
 #include "pk-shared.h"
 
-#include "pk-cleanup.h"
 #include "pk-transaction-db.h"
 
 static void     pk_transaction_db_finalize	(GObject        *object);
@@ -213,8 +212,8 @@ pk_transaction_db_action_time_since (PkTransactionDb *tdb, PkRoleEnum role)
 	gchar *error_msg = NULL;
 	gint rc;
 	const gchar *role_text;
-	_cleanup_free_ gchar *statement = NULL;
-	_cleanup_free_ gchar *timespec = NULL;
+	g_autofree gchar *statement = NULL;
+	g_autofree gchar *timespec = NULL;
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), 0);
 	g_return_val_if_fail (tdb->priv->db != NULL, 0);
@@ -246,8 +245,8 @@ pk_transaction_db_action_time_reset (PkTransactionDb *tdb, PkRoleEnum role)
 	gint rc;
 	const gchar *role_text;
 	guint since;
-	_cleanup_free_ gchar *statement = NULL;
-	_cleanup_free_ gchar *timespec = NULL;
+	g_autofree gchar *statement = NULL;
+	g_autofree gchar *timespec = NULL;
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), FALSE);
 	g_return_val_if_fail (tdb->priv->db != NULL, FALSE);
@@ -282,7 +281,7 @@ pk_transaction_db_get_list (PkTransactionDb *tdb, guint limit)
 	gchar *error_msg = NULL;
 	gint rc;
 	GList *list = NULL;
-	_cleanup_free_ gchar *statement = NULL;
+	g_autofree gchar *statement = NULL;
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), NULL);
 
@@ -313,8 +312,8 @@ out:
 gboolean
 pk_transaction_db_add (PkTransactionDb *tdb, const gchar *tid)
 {
-	_cleanup_free_ gchar *timespec = NULL;
-	_cleanup_free_ gchar *statement = NULL;
+	g_autofree gchar *timespec = NULL;
+	g_autofree gchar *statement = NULL;
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), FALSE);
 
@@ -331,7 +330,7 @@ gboolean
 pk_transaction_db_set_role (PkTransactionDb *tdb, const gchar *tid, PkRoleEnum role)
 {
 	const gchar *role_text;
-	_cleanup_free_ gchar *statement = NULL;
+	g_autofree gchar *statement = NULL;
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), FALSE);
 
@@ -347,7 +346,7 @@ pk_transaction_db_set_role (PkTransactionDb *tdb, const gchar *tid, PkRoleEnum r
 gboolean
 pk_transaction_db_set_uid (PkTransactionDb *tdb, const gchar *tid, guint uid)
 {
-	_cleanup_free_ gchar *statement = NULL;
+	g_autofree gchar *statement = NULL;
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), FALSE);
 
@@ -362,7 +361,7 @@ pk_transaction_db_set_uid (PkTransactionDb *tdb, const gchar *tid, guint uid)
 gboolean
 pk_transaction_db_set_cmdline (PkTransactionDb *tdb, const gchar *tid, const gchar *cmdline)
 {
-	_cleanup_free_ gchar *statement = NULL;
+	g_autofree gchar *statement = NULL;
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), FALSE);
 
@@ -377,7 +376,7 @@ pk_transaction_db_set_cmdline (PkTransactionDb *tdb, const gchar *tid, const gch
 gboolean
 pk_transaction_db_set_data (PkTransactionDb *tdb, const gchar *tid, const gchar *data)
 {
-	_cleanup_free_ gchar *statement = NULL;
+	g_autofree gchar *statement = NULL;
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), FALSE);
 
@@ -396,7 +395,7 @@ pk_transaction_db_set_data (PkTransactionDb *tdb, const gchar *tid, const gchar
 gboolean
 pk_transaction_db_set_finished (PkTransactionDb *tdb, const gchar *tid, gboolean success, guint runtime)
 {
-	_cleanup_free_ gchar *statement = NULL;
+	g_autofree gchar *statement = NULL;
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), FALSE);
 
@@ -485,7 +484,7 @@ pk_transaction_db_defer_write_job_count_cb (PkTransactionDb *tdb)
 {
 	gchar *error_msg = NULL;
 	gint rc;
-	_cleanup_free_ gchar *statement = NULL;
+	g_autofree gchar *statement = NULL;
 
 	/* not loaded! */
 	if (tdb->priv->db == NULL) {
@@ -520,7 +519,7 @@ gchar *
 pk_transaction_db_generate_id (PkTransactionDb *tdb)
 {
 	gchar *tid = NULL;
-	_cleanup_free_ gchar *rand_str = NULL;
+	g_autofree gchar *rand_str = NULL;
 
 	/* increment */
 	tdb->priv->job_count++;
@@ -605,7 +604,7 @@ pk_transaction_db_proxy_item_free (PkTransactionDbProxyItem *item)
  *
  * Retrieves the proxy information from the database.
  *
- * Return value: %TRUE if we matched a proxy
+ * Return value: %TRUE on success.
  **/
 gboolean
 pk_transaction_db_get_proxy (PkTransactionDb *tdb, guint uid, const gchar *session,
@@ -620,7 +619,7 @@ pk_transaction_db_get_proxy (PkTransactionDb *tdb, guint uid, const gchar *sessi
 	gboolean ret = FALSE;
 	gint rc;
 	PkTransactionDbProxyItem *item;
-	_cleanup_free_ gchar *statement = NULL;
+	g_autofree gchar *statement = NULL;
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), FALSE);
 	g_return_val_if_fail (uid != G_MAXUINT, FALSE);
@@ -639,13 +638,13 @@ pk_transaction_db_get_proxy (PkTransactionDb *tdb, guint uid, const gchar *sessi
 		goto out;
 	}
 
+	/* success, even if we got no data */
+	ret = TRUE;
+
 	/* nothing matched */
 	if (!item->set)
 		goto out;
 
-	/* success, even if we got no data */
-	ret = TRUE;
-
 	/* copy data */
 	if (proxy_http != NULL)
 		*proxy_http = g_strdup (item->proxy_http);
@@ -659,6 +658,7 @@ pk_transaction_db_get_proxy (PkTransactionDb *tdb, guint uid, const gchar *sessi
 		*no_proxy = g_strdup (item->no_proxy);
 	if (pac != NULL)
 		*pac = g_strdup (item->pac);
+
 out:
 	pk_transaction_db_proxy_item_free (item);
 	return ret;
@@ -689,8 +689,8 @@ pk_transaction_db_set_proxy (PkTransactionDb *tdb, guint uid,
 	gboolean ret = FALSE;
 	gint rc;
 	sqlite3_stmt *statement = NULL;
-	_cleanup_free_ gchar *timespec = NULL;
-	_cleanup_free_ gchar *proxy_http_tmp = NULL;
+	g_autofree gchar *timespec = NULL;
+	g_autofree gchar *proxy_http_tmp = NULL;
 
 	g_return_val_if_fail (PK_IS_TRANSACTION_DB (tdb), FALSE);
 	g_return_val_if_fail (uid != G_MAXUINT, FALSE);
@@ -805,7 +805,7 @@ pk_transaction_db_class_init (PkTransactionDbClass *klass)
 static void
 pk_transaction_db_ensure_file_directory (const gchar *path)
 {
-	_cleanup_free_ gchar *parent = NULL;
+	g_autofree gchar *parent = NULL;
 	parent = g_path_get_dirname (path);
 	if (g_mkdir_with_parents (parent, 0755) == -1)
 		g_warning ("%s", g_strerror (errno));
